/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

package config

import (
	"github.com/pkg/errors"

	"huawei.com/huaweichain/proto/shard"
)

// ShardConfig is the definition of shard config.
type ShardConfig struct {
	ShardingNodes []ShardNode
	Shards        map[string][]string
}

// NewShardConfig is used create an instance of shard config.
func NewShardConfig(configPath string) (*ShardConfig, error) {
	v, errMsg := parseCfg(configPath)
	if errMsg != nil {
		return nil, errors.WithMessage(errMsg, "parse config file error")
	}

	shardConfig := &ShardConfig{}
	errMsg = unmarshalShardConfig(v, &shardConfig)
	if errMsg != nil {
		return nil, errors.WithMessage(errMsg, "error unmarshaling config into struct")
	}
	return shardConfig, nil
}

// GetShardInfo is used to get shard info.
func GetShardInfo(config *ShardConfig) (*shard.ShardInfo, error) {
	info := &shard.ShardInfo{
		Shards: make(map[string]*shard.ShardNodeList),
	}
	shardNodeMap := make(map[string]*shard.ShardNode)
	for idx := range config.ShardingNodes {
		node := &shard.ShardNode{
			Host: config.ShardingNodes[idx].Host,
			Port: config.ShardingNodes[idx].Port,
		}
		shardNodeMap[config.ShardingNodes[idx].ID] = node
	}
	for shardID, nodeIDs := range config.Shards {
		shardNodeList := &shard.ShardNodeList{}
		for _, nodeID := range nodeIDs {
			shardNodeList.ShardNodes = append(shardNodeList.ShardNodes, shardNodeMap[nodeID])
		}
		info.Shards[shardID] = shardNodeList
	}
	return info, nil
}
