/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package flow

// IdentityFlow is used to set identity config
type IdentityFlow struct {
	Flow
	decrypt func([]byte) []byte
}

func newIdentityFlow(stub *stub) *IdentityFlow {
	return &IdentityFlow{
		Flow: Flow{stub: stub},
		decrypt: func(bytes []byte) []byte {
			return nil
		},
	}
}

// Identity is used to set identity client config
func (i *IdentityFlow) Identity(algorithm string, key []byte, cert []byte) *IdentityFlow {
	i.stub.SetIdentity(algorithm, i.decrypt(key), i.decrypt(cert))
	return i
}

// SetDecrypt is used to set decrypt method
func (i *IdentityFlow) SetDecrypt(decrypt func([]byte) []byte) *IdentityFlow {
	i.decrypt = decrypt
	return i
}
