/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package flow

import (
	"github.com/pkg/errors"

	"huawei.com/huaweichain/sdk/node"
)

// OperatorFlow is the role of operator
// which is to invoke config/lifecycle flow streamly
type OperatorFlow struct {
	*Flow
	node *node.WNode
}

func newOperatorFlow(stub *stub) *OperatorFlow {
	return &OperatorFlow{
		Flow: &Flow{stub: stub},
	}
}

// CheckNodeExisted is used to check node existed
func (o *OperatorFlow) CheckNodeExisted(node string) bool {
	return o.stub.gatewayClient.Nodes[node] != nil
}

// SetNode is used to set node
func (o *OperatorFlow) SetNode(node string) (*OperatorFlow, error) {
	n, ok := o.stub.gatewayClient.Nodes[node]
	if !ok {
		return nil, errors.Wrap(ErrNoSuchNode, "failed to set node for operator flow")
	}
	o.node = n
	return o, nil
}

// SetCheckedNode is used to set node without error return
func (o *OperatorFlow) SetCheckedNode(node string) *OperatorFlow {
	_, err := o.SetNode(node)
	if err != nil {
		log.Fatal(err.Error())
	}
	return o
}

// Config is used to return ConfigFlow
func (o *OperatorFlow) Config() *ConfigFlow {
	return newConfigFlow(o.stub, o.node)
}

// Lifecycle is used to return LifecycleFlow
func (o *OperatorFlow) Lifecycle() *LifecycleFlow {
	return newLifecycleFlow(o.stub)
}
