/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package flow

import (
	"math/rand"
	"time"

	"github.com/pkg/errors"

	"huawei.com/huaweichain/proto/common"
)

type performerFlow struct {
	*ObserverFlow
}

func (p *performerFlow) randomNode(nodeList []string) (string, error) {
	length := len(nodeList)
	if length == 0 {
		return "", errors.New("nodeList is null")
	}
	rand.Seed(time.Now().UnixNano())
	randomIndex := rand.Intn(length) // nolint:gosec // not security case
	if check := p.CheckNodeConnected(nodeList[randomIndex]); check {
		return nodeList[randomIndex], nil
	}
	if length <= 1 {
		return "", errors.New("fail to select nodes in method randomNode")
	}
	nodeList = append(nodeList[:randomIndex], nodeList[randomIndex+1:]...)
	return p.randomNode(nodeList)
}

// CheckNodeConnected is used to check node connection state by send a invoke message
func (p *performerFlow) CheckNodeConnected(nodeName string) bool {
	rawMes := &common.RawMessage{}
	node, ok := p.Flow.stub.gatewayClient.Nodes[nodeName]
	if !ok {
		return false
	}
	_, err := node.ContractAction.Invoke(rawMes)
	return err == nil
}
