/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package flow

import (
	"fmt"

	"github.com/pkg/errors"

	"huawei.com/huaweichain/sdk/client"
	"huawei.com/huaweichain/sdk/crypto"
	"huawei.com/huaweichain/sdk/node"
	"huawei.com/huaweichain/sdk/rawmessage"
)

// stub is used to cover gatewayClient
type stub struct {
	gatewayClient *client.GatewayClient
}

func newStub() *stub {
	return &stub{
		gatewayClient: client.NewEmptyGatewayClient(),
	}
}

// SetIdentity is used to call the same method in gatewayClient
func (s *stub) SetIdentity(algorithm string, key []byte, cert []byte) {
	err := s.gatewayClient.SetIdentity(algorithm, key, cert)
	if err != nil {
		log.Fatal(err.Error())
	}
}

// SetTLS is used to call the same method in gatewayClient
func (s *stub) SetTLS(certPEMBlock []byte, keyPEMBlock []byte, roots [][]byte) {
	s.gatewayClient.SetTLS(certPEMBlock, keyPEMBlock, roots)
}

// AddNode is used to call the same method in gatewayClient
func (s *stub) AddNode(name string, hostOverride string, host string, port int) {
	err := s.gatewayClient.AddNode(name, hostOverride, host, port)
	if err != nil {
		log.Fatal(err.Error())
	}
}

// GetNode is used to transform the struct node
func (s *stub) GetNode(name string) Node {
	nodeMap := s.gatewayClient.Nodes
	n := nodeMap[name]
	newNode := NewNode(n.ID, n.Host, n.HostOverride, n.Port)
	return *newNode
}

func (s *stub) GetWNode(name string) (*node.WNode, error) {
	nodeMap := s.gatewayClient.Nodes
	n, ok := nodeMap[name]
	if !ok {
		return nil, errors.Errorf("node %s doesn't exist", name)
	}
	return n, nil
}

func (s *stub) WNode(name string) *node.WNode {
	nodeMap := s.gatewayClient.Nodes
	n, ok := nodeMap[name]
	if !ok {
		panic(fmt.Sprintf("node %s doesn't exist", name))
	}
	return n
}

// GetChainRawMessage is used to get chain raw message
func (s *stub) GetChainRawMessage() *rawmessage.ChainRawMessage {
	return s.gatewayClient.ChainRawMessage
}

// GetQueryRawMessage is used to get query raw message
func (s *stub) GetQueryRawMessage() *rawmessage.QueryRawMessage {
	return s.gatewayClient.QueryRawMessage
}

// GetContractRawMessage is used to get contract raw message
func (s *stub) GetContractRawMessage() *rawmessage.ContractRawMessage {
	return s.gatewayClient.ContractRawMessage
}

// GetLifecycleRawMessage is used to get lifecycle raw message
func (s *stub) GetLifecycleRawMessage() *rawmessage.LifecycleRawMessage {
	return s.gatewayClient.LifecycleRawMessage
}

// GetConfigRawMessage is used to get config raw message
func (s *stub) GetConfigRawMessage() *rawmessage.ConfigRawMessage {
	return s.gatewayClient.ConfigRawMessage
}

func (s *stub) GetCrypto() crypto.Crypto {
	return s.gatewayClient.Crypto()
}
