/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package flow

// TlSFlow is used to set tls config
type TlSFlow struct {
	Flow
	decrypt func([]byte) []byte
	roots   [][]byte
}

func newTLSFlow(stub *stub) *TlSFlow {
	tlsFlowInstance := TlSFlow{
		Flow: Flow{
			stub: stub,
		},
		decrypt: func(bytes []byte) []byte {
			return nil
		},
	}
	return &tlsFlowInstance
}

// SetRoots is used to set ca certs for sdk
func (t *TlSFlow) SetRoots(roots [][]byte) *TlSFlow {
	t.roots = roots
	return t
}

// SetDecrypt is used to set decrypt for sdk
func (t *TlSFlow) SetDecrypt(decrypt func([]byte) []byte) *TlSFlow {
	t.decrypt = decrypt
	return t
}

// KeyPair is used to set keys and certs for sdk
func (t *TlSFlow) KeyPair(key []byte, cert []byte) *TlSFlow {
	caCerts := make([][]byte, 0, len(t.roots))
	for i := 0; i < len(t.roots); i++ {
		caCerts = append(caCerts, t.decrypt(t.roots[i]))
	}
	t.stub.SetTLS(t.decrypt(cert), t.decrypt(key), caCerts)
	return t
}
