/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package flow

import (
	"encoding/hex"
	"encoding/json"
	"fmt"
	"regexp"

	"github.com/pkg/errors"

	"huawei.com/huaweichain/proto/common"
)

// InfoOptions is the definition of InfoOptions.
type InfoOptions struct {
	Node   string `json:"node"`
	Status string `json:"status"`
	Info   string `json:"info"`
	TxID   string `json:"tx_id,omitempty"`
}

const nodeReg = "^node([0-9]+)_org([0-9]+)$"

// PrintResponse is used to print response.
func PrintResponse(response *common.Response, node string, info string, opts ...Option) {
	if response.Status == common.SUCCESS {
		PrintInfo(node, response.Status.String(), info, opts...)
	} else {
		PrintInfo(node, response.Status.String(), response.StatusInfo, opts...)
	}
}

// Option is the interface for set field of info options.
type Option func(*InfoOptions)

// PrintInfo is used to print info with json format.
func PrintInfo(node string, status string, info string, opts ...Option) {
	options := NewInfoOptions(node, status, info)
	if len(opts) > 0 {
		for _, opt := range opts {
			opt(options)
		}
	}

	b, err := json.Marshal(options)
	if err != nil {
		log.Errorf("marshal json error: %v", err)
	}
	fmt.Println(string(b))
}

// NewInfoOptions is used to new an instance of InfoOptions.
func NewInfoOptions(node string, status string, info string) *InfoOptions {
	return &InfoOptions{Node: node, Status: status, Info: info}
}

// Hash2str is used to convert the hash value to string for human reading.
func Hash2str(h []byte) string {
	return hex.EncodeToString(h)
}

func checkNodeReg(node string) (bool, error) {
	r, err := regexp.Compile(nodeReg)
	if err != nil {
		return false, errors.New("compile error")
	}
	if match := r.MatchString(node); !match {
		return false, nil
	}
	return true, nil
}
